use crate::event::Event;

/// Interface for filtering an `InternalEvent`.
pub(crate) trait Filter: Send + Sync + 'static {
    /// Returns whether the given event fulfills the filter.
    fn eval(&self, event: &Event) -> bool;
}

#[derive(Debug, Clone)]
pub(crate) struct CursorPositionFilter;

impl Filter for CursorPositionFilter {
    fn eval(&self, event: &Event) -> bool {
        matches!(*event, Event::CursorPosition(_, _))
    }
}

#[derive(Debug, Clone)]
pub(crate) struct KeyboardEnhancementFlagsFilter;

impl Filter for KeyboardEnhancementFlagsFilter {
    fn eval(&self, event: &Event) -> bool {
        // This filter checks for either a KeyboardEnhancementFlags response or
        // a PrimaryDeviceAttributes response. If we receive the PrimaryDeviceAttributes
        // response but not KeyboardEnhancementFlags, the terminal does not support
        // progressive keyboard enhancement.
        matches!(
            *event,
            Event::KeyboardEnhancementFlags(_) | Event::PrimaryDeviceAttributes
        )
    }
}

#[derive(Debug, Clone)]
pub(crate) struct PrimaryDeviceAttributesFilter;

impl Filter for PrimaryDeviceAttributesFilter {
    fn eval(&self, event: &Event) -> bool {
        matches!(*event, Event::PrimaryDeviceAttributes)
    }
}

#[derive(Debug, Clone)]
pub(crate) struct EventFilter;

impl Filter for EventFilter {
    fn eval(&self, _: &Event) -> bool {
        true
    }
}

#[cfg(test)]
mod tests {
    use super::{
        super::Event, CursorPositionFilter, EventFilter, Filter, KeyboardEnhancementFlagsFilter,
        PrimaryDeviceAttributesFilter,
    };

    #[test]
    fn test_cursor_position_filter_filters_cursor_position() {
        assert!(!CursorPositionFilter.eval(&Event::Resize(10, 10)));
        assert!(CursorPositionFilter.eval(&Event::CursorPosition(0, 0)));
    }

    #[test]
    fn test_keyboard_enhancement_status_filter_filters_keyboard_enhancement_status() {
        assert!(!KeyboardEnhancementFlagsFilter.eval(&Event::Resize(10, 10)));
        assert!(
            KeyboardEnhancementFlagsFilter.eval(&Event::KeyboardEnhancementFlags(
                crate::event::KeyboardEnhancementFlags::DISAMBIGUATE_ESCAPE_CODES
            ))
        );
        assert!(KeyboardEnhancementFlagsFilter.eval(&Event::PrimaryDeviceAttributes));
    }

    #[test]
    fn test_primary_device_attributes_filter_filters_primary_device_attributes() {
        assert!(!PrimaryDeviceAttributesFilter.eval(&Event::Resize(10, 10)));
        assert!(PrimaryDeviceAttributesFilter.eval(&Event::PrimaryDeviceAttributes));
    }

    #[test]
    fn test_event_filter_filters_events() {
        assert!(EventFilter.eval(&Event::Resize(10, 10)));
        assert!(EventFilter.eval(&Event::CursorPosition(0, 0)));
    }
}
